@@ -1,11 +1,9 @@
-import json
-from urllib2 import Request, HTTPError
-from urllib import urlencode
+from requests import HTTPError
+from requests_oauthlib import OAuth1
+from oauthlib.oauth1 import SIGNATURE_TYPE_AUTH_HEADER
 
-from oauth2 import Token, SignatureMethod_HMAC_SHA1, HTTP_METHOD, \
-                   Request as OAuthRequest, Consumer as OAuthConsumer
-
-from social.utils import url_add_parameters
+from social.p3 import urlencode
+from social.utils import url_add_parameters, parse_qs
 from social.exceptions import AuthFailed, AuthCanceled, AuthUnknownError, \
                               AuthMissingParameter, AuthStateMissing, \
                               AuthStateForbidden, AuthTokenError
@@ -30,6 +28,7 @@ class OAuthAuth(BaseAuth):
     SCOPE_SEPARATOR = ' '
     EXTRA_DATA = None
     ID_KEY = 'id'
+    ACCESS_TOKEN_METHOD = 'GET'
 
     def extra_data(self, user, uid, response, details=None):
         """Return access_token and extra defined names to store in
@@ -38,19 +37,20 @@ class OAuthAuth(BaseAuth):
         names = (self.EXTRA_DATA or []) + \
                 self.setting('EXTRA_DATA', [])
         for entry in names:
-            if len(entry) == 2:
-                (name, alias), discard = entry, False
-            elif len(entry) == 3:
-                name, alias, discard = entry
-            elif len(entry) == 1:
-                name = alias = entry
-            else:  # ???
-                continue
-
-            value = response.get(name)
-            if discard and not value:
-                continue
-            data[alias] = value
+            if not isinstance(entry, (list, tuple)):
+                entry = (entry,)
+            size = len(entry)
+            if size >= 1 and size <= 3:
+                if size == 3:
+                    name, alias, discard = entry
+                elif size == 2:
+                    (name, alias), discard = entry, False
+                elif size == 1:
+                    name = alias = entry
+                value = response.get(name)
+                if discard and not value:
+                    continue
+                data[alias] = value
         return data
 
     def get_scope(self):
@@ -70,7 +70,7 @@ class OAuthAuth(BaseAuth):
         return {}
 
 
-class ConsumerBasedOAuth(OAuthAuth):
+class BaseOAuth1(OAuthAuth):
     """Consumer based mechanism OAuth authentication, fill the needed
     parameters to communicate properly with authentication service.
 
@@ -80,38 +80,54 @@ class ConsumerBasedOAuth(OAuthAuth):
     """
     AUTHORIZATION_URL = ''
     REQUEST_TOKEN_URL = ''
+    REQUEST_TOKEN_METHOD = 'GET'
+    OAUTH_TOKEN_PARAMETER_NAME = 'oauth_token'
+    REDIRECT_URI_PARAMETER_NAME = 'redirect_uri'
     ACCESS_TOKEN_URL = ''
 
     def auth_url(self):
         """Return redirect url"""
         token = self.unauthorized_token()
         name = self.name + 'unauthorized_token_name'
-        tokens = self.strategy.session_get(name, [])
-        tokens.append(token.to_string())
+        tokens = self.strategy.session_get(name, []) + [token]
         self.strategy.session_set(name, tokens)
-        return self.oauth_authorization_request(token).to_url()
+        return self.oauth_authorization_request(token)
+
+    def process_error(self, data):
+        if 'oauth_problem' in data:
+            if data['oauth_problem'] == 'user_refused':
+                raise AuthCanceled(self, 'User refused the access')
+            raise AuthUnknownError(self, 'Error was ' + data['oauth_problem'])
 
     def auth_complete(self, *args, **kwargs):
         """Return user, might be logged in"""
         # Multiple unauthorized tokens are supported (see #521)
+        self.process_error(self.data)
         name = self.name + 'unauthorized_token_name'
         token = None
         unauthed_tokens = self.strategy.session_get(name, [])
         if not unauthed_tokens:
             raise AuthTokenError(self, 'Missing unauthorized token')
+        token_param_name = self.OAUTH_TOKEN_PARAMETER_NAME
+        data_token = self.data.get(token_param_name, 'no-token')
         for unauthed_token in unauthed_tokens:
-            token = Token.from_string(unauthed_token)
-            if token.key == self.data.get('oauth_token', 'no-token'):
-                self.strategy.session_set(name, list(set(unauthed_tokens) -
-                                                     set([unauthed_token])))
+            orig_unauthed_token = unauthed_token
+            if not isinstance(unauthed_token, dict):
+                unauthed_token = parse_qs(unauthed_token)
+            if unauthed_token.get(token_param_name) == data_token:
+                self.strategy.session_set(name, list(
+                    set(unauthed_tokens) -
+                    set([orig_unauthed_token]))
+                )
+                token = unauthed_token
                 break
         else:
             raise AuthTokenError(self, 'Incorrect tokens')
 
         try:
             access_token = self.access_token(token)
-        except HTTPError, e:
-            if e.code == 400:
+        except HTTPError as err:
+            if err.response.status_code == 400:
                 raise AuthCanceled(self)
             else:
                 raise
@@ -120,53 +136,57 @@ class ConsumerBasedOAuth(OAuthAuth):
     def do_auth(self, access_token, *args, **kwargs):
         """Finish the auth process once the access_token was retrieved"""
         data = self.user_data(access_token)
-        if data is not None:
-            data['access_token'] = access_token.to_string()
+        if data is not None and 'access_token' not in data:
+            data['access_token'] = access_token
         kwargs.update({'response': data, 'backend': self})
         return self.strategy.authenticate(*args, **kwargs)
 
     def unauthorized_token(self):
         """Return request for unauthorized token (first stage)"""
-        request = self.oauth_request(
-            token=None,
-            url=self.REQUEST_TOKEN_URL,
-            extra_params=self.request_token_extra_arguments()
-        )
-        return Token.from_string(self.fetch_response(request))
+        params = self.request_token_extra_arguments()
+        params.update(self.get_scope_argument())
+        key, secret = self.get_key_and_secret()
+        response = self.request(self.REQUEST_TOKEN_URL,
+                                params=params,
+                                auth=OAuth1(key, secret,
+                                            callback_uri=self.redirect_uri),
+                                method=self.REQUEST_TOKEN_METHOD)
+        return response.content
 
     def oauth_authorization_request(self, token):
         """Generate OAuth request to authorize token."""
+        if not isinstance(token, dict):
+            token = parse_qs(token)
         params = self.auth_extra_arguments() or {}
         params.update(self.get_scope_argument())
-        return OAuthRequest.from_token_and_callback(
-            token=token,
-            callback=self.redirect_uri,
-            http_url=self.AUTHORIZATION_URL,
-            parameters=params
+        params[self.OAUTH_TOKEN_PARAMETER_NAME] = token.get(
+            self.OAUTH_TOKEN_PARAMETER_NAME
         )
+        params[self.REDIRECT_URI_PARAMETER_NAME] = self.redirect_uri
+        return self.AUTHORIZATION_URL + '?' + urlencode(params)
 
-    def oauth_request(self, token, url, extra_params=None, method=HTTP_METHOD):
+    def oauth_auth(self, token=None, oauth_verifier=None,
+                   signature_type=SIGNATURE_TYPE_AUTH_HEADER):
+        key, secret = self.get_key_and_secret()
+        oauth_verifier = oauth_verifier or self.data.get('oauth_verifier')
+        token = token or {}
+        return OAuth1(key, secret,
+                      resource_owner_key=token.get('oauth_token'),
+                      resource_owner_secret=token.get('oauth_token_secret'),
+                      callback_uri=self.redirect_uri,
+                      verifier=oauth_verifier,
+                      signature_type=signature_type)
+
+    def oauth_request(self, token, url, params=None, method='GET'):
         """Generate OAuth request, setups callback url"""
-        return build_consumer_oauth_request(self, token, url,
-                                            self.redirect_uri,
-                                            self.data.get('oauth_verifier'),
-                                            extra_params,
-                                            method=method)
-
-    def fetch_response(self, request):
-        """Executes request and fetchs service response"""
-        response = self.urlopen(request.to_url())
-        return '\n'.join(response.readlines())
+        return self.request(url, method=method, params=params,
+                            auth=self.oauth_auth(token))
 
     def access_token(self, token):
         """Return request for access token value"""
-        request = self.oauth_request(token, self.ACCESS_TOKEN_URL)
-        return Token.from_string(self.fetch_response(request))
-
-    @property
-    def consumer(self):
-        """Setups consumer"""
-        return OAuthConsumer(*self.get_key_and_secret())
+        return self.get_querystring(self.ACCESS_TOKEN_URL,
+                                    auth=self.oauth_auth(token),
+                                    method=self.ACCESS_TOKEN_METHOD)
 
 
 class BaseOAuth2(OAuthAuth):
@@ -182,6 +202,7 @@ class BaseOAuth2(OAuthAuth):
     AUTHORIZATION_URL = None
     ACCESS_TOKEN_URL = None
     REFRESH_TOKEN_URL = None
+    REFRESH_TOKEN_METHOD = 'POST'
     RESPONSE_TYPE = 'code'
     REDIRECT_STATE = True
     STATE_PARAMETER = True
@@ -247,11 +268,6 @@ class BaseOAuth2(OAuthAuth):
                 raise AuthStateForbidden(self)
         return state
 
-    def process_error(self, data):
-        if data.get('error'):
-            raise AuthFailed(self, self.data.get('error_description') or
-                                   self.data['error'])
-
     def auth_complete_params(self, state=None):
         client_id, client_secret = self.get_key_and_secret()
         return {
@@ -266,21 +282,34 @@ class BaseOAuth2(OAuthAuth):
         return {'Content-Type': 'application/x-www-form-urlencoded',
                 'Accept': 'application/json'}
 
+    def request_access_token(self, *args, **kwargs):
+        return self.get_json(*args, **kwargs)
+
+    def process_error(self, data):
+        if data.get('error'):
+            if data['error'] == 'denied' or data['error'] == 'access_denied':
+                raise AuthCanceled(self, data.get('error_description', ''))
+            raise AuthFailed(self, data.get('error_description') or
+                                   data['error'])
+        elif 'denied' in data:
+            raise AuthCanceled(self, data['denied'])
+
     def auth_complete(self, *args, **kwargs):
         """Completes loging process, must return user instance"""
         self.process_error(self.data)
-        params = self.auth_complete_params(self.validate_state())
-        request = Request(self.ACCESS_TOKEN_URL, data=urlencode(params),
-                          headers=self.auth_headers())
-
         try:
-            response = json.loads(self.urlopen(request).read())
-        except HTTPError, e:
-            if e.code == 400:
+            response = self.request_access_token(
+                self.ACCESS_TOKEN_URL,
+                data=self.auth_complete_params(self.validate_state()),
+                headers=self.auth_headers(),
+                method=self.ACCESS_TOKEN_METHOD
+            )
+        except HTTPError as err:
+            if err.response.status_code == 400:
                 raise AuthCanceled(self)
             else:
                 raise
-        except (ValueError, KeyError):
+        except KeyError:
             raise AuthUnknownError(self)
         self.process_error(response)
         return self.do_auth(response['access_token'], response=response,
@@ -294,7 +323,7 @@ class BaseOAuth2(OAuthAuth):
         kwargs.update({'response': response, 'backend': self})
         return self.strategy.authenticate(*args, **kwargs)
 
-    def refresh_token_params(self, token):
+    def refresh_token_params(self, token, *args, **kwargs):
         client_id, client_secret = self.get_key_and_secret()
         return {
             'refresh_token': token,
@@ -303,36 +332,12 @@ class BaseOAuth2(OAuthAuth):
             'client_secret': client_secret
         }
 
-    def process_refresh_token_response(self, response):
-        return json.loads(response)
+    def process_refresh_token_response(self, response, *args, **kwargs):
+        return response.json()
 
-    def refresh_token(self, token):
-        request = Request(
-            self.REFRESH_TOKEN_URL or self.ACCESS_TOKEN_URL,
-            data=urlencode(self.refresh_token_params(token)),
-            headers=self.auth_headers()
-        )
-        return self.process_refresh_token_response(
-            self.urlopen(request).read()
-        )
-
-
-def build_consumer_oauth_request(backend, token, url, redirect_uri='/',
-                                 oauth_verifier=None, extra_params=None,
-                                 method=HTTP_METHOD):
-    """Builds a Consumer OAuth request."""
-    params = {'oauth_callback': redirect_uri}
-    if extra_params:
-        params.update(extra_params)
-
-    if oauth_verifier:
-        params['oauth_verifier'] = oauth_verifier
-
-    consumer = OAuthConsumer(*backend.get_key_and_secret())
-    request = OAuthRequest.from_consumer_and_token(consumer,
-                                                   token=token,
-                                                   http_method=method,
-                                                   http_url=url,
-                                                   parameters=params)
-    request.sign_request(SignatureMethod_HMAC_SHA1(), consumer, token)
-    return request
+    def refresh_token(self, token, *args, **kwargs):
+        params = self.refresh_token_params(token, *args, **kwargs)
+        request = self.request(self.REFRESH_TOKEN_URL or self.ACCESS_TOKEN_URL,
+                               params=params, headers=self.auth_headers(),
+                               method=self.REFRESH_TOKEN_METHOD)
+        return self.process_refresh_token_response(request, *args, **kwargs)
