@@ -10,17 +10,16 @@ Then update your settings values using registration information
 ref:
     https://github.com/gkmngrgn/django-tumblr-auth
 """
-import json
-
 from social.utils import first
-from social.backends.oauth import ConsumerBasedOAuth
+from social.backends.oauth import BaseOAuth1
 
 
-class TumblrOAuth(ConsumerBasedOAuth):
+class TumblrOAuth(BaseOAuth1):
     name = 'tumblr'
     ID_KEY = 'username'
     AUTHORIZATION_URL = 'http://www.tumblr.com/oauth/authorize'
     REQUEST_TOKEN_URL = 'http://www.tumblr.com/oauth/request_token'
+    REQUEST_TOKEN_METHOD = 'POST'
     ACCESS_TOKEN_URL = 'http://www.tumblr.com/oauth/access_token'
 
     def get_user_details(self, response):
@@ -33,24 +32,5 @@ class TumblrOAuth(ConsumerBasedOAuth):
         return data
 
     def user_data(self, access_token):
-        request = self.oauth_request(access_token,
-                                     'http://api.tumblr.com/v2/user/info')
-        try:
-            return json.loads(self.fetch_response(request))
-        except ValueError:
-            return None
-
-    @classmethod
-    def tokens(cls, instance):
-        """
-        Return the tokens needed to authenticate the access to any API the
-        service might provide. Tumblr uses a pair of OAuthToken consisting
-        on a oauth_token and oauth_token_secret.
-
-        instance must be a UserSocialAuth instance.
-        """
-        token = super(TumblrOAuth, cls).tokens(instance)
-        if token and 'access_token' in token:
-            token = dict(tok.split('=')
-                            for tok in token['access_token'].split('&'))
-        return token
+        return self.get_json('http://api.tumblr.com/v2/user/info',
+                             auth=self.oauth_auth(access_token))
