@@ -10,7 +10,9 @@ def social_user(strategy, uid, user=None, *args, **kwargs):
             raise AuthAlreadyAssociated(strategy.backend, msg)
         elif not user:
             user = social_user.user
-    return {'social_user': social_user, 'user': user}
+    return {'social_user': social_user,
+            'user': user,
+            'new_association': False}
 
 
 def associate_user(strategy, user, uid, social_user=None, *args, **kwargs):
@@ -21,8 +23,8 @@ def associate_user(strategy, user, uid, social_user=None, *args, **kwargs):
         social = strategy.storage.user.create_social_auth(
             user, uid, strategy.backend.name
         )
-    except Exception, e:
-        if not strategy.is_integrity_error(e):
+    except Exception as err:
+        if not strategy.is_integrity_error(err):
             raise
         # Protect for possible race condition, those bastard with FTL
         # clicking capabilities, check issue #131:
@@ -30,7 +32,9 @@ def associate_user(strategy, user, uid, social_user=None, *args, **kwargs):
         return social_user(strategy.backend, uid, user,
                            social_user=social_user, *args, **kwargs)
     else:
-        return {'social_user': social, 'user': social.user}
+        return {'social_user': social,
+                'user': social.user,
+                'new_association': True}
 
 
 def load_extra_data(strategy, details, response, uid, user, social_user=None,
