@@ -6,22 +6,47 @@ from social.utils import setting_name
 from social.store import OpenIdStore
 
 
+class BaseTemplateStrategy(object):
+    def __init__(self, strategy):
+        self.strategy = strategy
+
+    def render(self, tpl=None, html=None, context=None):
+        if not tpl and not html:
+            raise ValueError('Missing template or html parameters')
+        context = context or {}
+        if tpl:
+            return self.render_template(tpl, **context)
+        else:
+            return self.render_string(html, **context)
+
+    def render_template(self, tpl, context):
+        raise NotImplemented('Implement in subclass')
+
+    def render_string(self, html, context):
+        raise NotImplemented('Implement in subclass')
+
+
 class BaseStrategy(object):
     ALLOWED_CHARS = 'abcdefghijklmnopqrstuvwxyz' \
                     'ABCDEFGHIJKLMNOPQRSTUVWXYZ' \
                     '0123456789'
 
-    def __init__(self, backend=None, storage=None, request=None,
-                 *args, **kwargs):
+    def __init__(self, backend=None, storage=None, request=None, tpl=None,
+                 backends=None, *args, **kwargs):
+        if not isinstance(tpl, BaseTemplateStrategy):
+            tpl = tpl(self)
+        self.tpl = tpl
         self.request = request
         self.storage = storage
+        self.backends = backends
         if backend:
+            self.backend_name = backend.name
             self.backend = backend(strategy=self, *args, **kwargs)
         else:
+            self.backend_name = None
             self.backend = backend
 
     def setting(self, name, default=None):
-        setting = default
         names = (setting_name(self.backend.titled_name, name),
                  setting_name(name),
                  name)
@@ -30,7 +55,7 @@ class BaseStrategy(object):
                 return self.get_setting(name)
             except (AttributeError, KeyError):
                 pass
-        return setting
+        return default
 
     def start(self):
         # Clean any partial pipeline info before starting the process
@@ -75,9 +100,8 @@ class BaseStrategy(object):
         }
 
     def from_session(self, session):
-        saved_args = session['args']
-        saved_kwargs = session['kwargs']
-        return session['next'], saved_args, saved_kwargs
+        return session['next'], session['backend'], \
+               session['args'], session['kwargs']
 
     def clean_partial_pipeline(self):
         self.session_pop('partial_pipeline')
@@ -125,9 +149,9 @@ class BaseStrategy(object):
 
     def render_html(self, tpl=None, html=None, context=None):
         """Render given template or raw html with given context"""
-        raise NotImplementedError('Implement in subclass')
+        return self.tpl.render(tpl, html, context)
 
-    def request_data(self):
+    def request_data(self, merge=True):
         """Return current request data (POST or GET)"""
         raise NotImplementedError('Implement in subclass')
 
@@ -135,7 +159,7 @@ class BaseStrategy(object):
         """Return current host value"""
         raise NotImplementedError('Implement in subclass')
 
-    def session_get(self, name):
+    def session_get(self, name, default=None):
         """Return session value for given key"""
         raise NotImplementedError('Implement in subclass')
 
@@ -150,3 +174,6 @@ class BaseStrategy(object):
     def build_absolute_uri(self, path=None):
         """Build absolute URI with given (optional) path"""
         raise NotImplementedError('Implement in subclass')
+
+    def is_response(self, value):
+        raise NotImplemented('Implement in subclass')
