@@ -1,10 +1,11 @@
 import re
 import sys
-import urlparse
-import urllib
 import unicodedata
-from cgi import parse_qsl
-from datetime import timedelta, tzinfo
+import six
+import collections
+
+from social.p3 import urlparse, urlunparse, urlencode, \
+                      parse_qs as battery_parse_qs
 
 
 SETTING_PREFIX = 'SOCIAL_AUTH'
@@ -24,30 +25,14 @@ def module_member(name):
 def url_add_parameters(url, params):
     """Adds parameters to URL, parameter will be repeated if already present"""
     if params:
-        fragments = list(urlparse.urlparse(url))
-        fragments[4] = urllib.urlencode(parse_qsl(fragments[4]) +
-                                        params.items())
-        url = urlparse.urlunparse(fragments)
+        fragments = list(urlparse(url))
+        value = parse_qs(fragments[4])
+        value.update(params)
+        fragments[4] = urlencode(value)
+        url = urlunparse(fragments)
     return url
 
 
-class UTC(tzinfo):
-    """UTC implementation taken from django 1.4."""
-    def __repr__(self):
-        return '<UTC>'
-
-    def utcoffset(self, dt):
-        return timedelta(0)
-
-    def tzname(self, dt):
-        return 'UTC'
-
-    def dst(self, dt):
-        return timedelta(0)
-
-utc = UTC()
-
-
 def setting_name(*names):
     return '_'.join((SETTING_PREFIX,) + tuple(names))
 
@@ -58,29 +43,13 @@ def sanitize_redirect(host, redirect_to):
     this method tests it to make sure it isn't garbage/harmful
     and returns it, else returns None, similar as how's it done
     on django.contrib.auth.views.
-
-    >>> print sanitize_redirect('myapp.com', None)
-    None
-    >>> print sanitize_redirect('myapp.com', '')
-    None
-    >>> print sanitize_redirect('myapp.com', {})
-    None
-    >>> print sanitize_redirect('myapp.com', 'http://notmyapp.com/path/')
-    None
-    >>> print sanitize_redirect('myapp.com', 'http://myapp.com/path/')
-    http://myapp.com/path/
-    >>> print sanitize_redirect('myapp.com', '/path/')
-    /path/
     """
     # Quick sanity check.
-    if not redirect_to:
+    if not redirect_to or not isinstance(redirect_to, six.string_types):
         return None
 
     # Heavier security check, don't allow redirection to a different host.
-    try:
-        netloc = urlparse(redirect_to)[1]
-    except TypeError:  # not valid redirect_to value
-        return None
+    netloc = urlparse(redirect_to)[1]
     if netloc and netloc != host:
         return None
     return redirect_to
@@ -88,7 +57,7 @@ def sanitize_redirect(host, redirect_to):
 
 def user_is_authenticated(user):
     if user and hasattr(user, 'is_authenticated'):
-        if callable(user.is_authenticated):
+        if isinstance(user.is_authenticated, collections.Callable):
             authenticated = user.is_authenticated()
         else:
             authenticated = user.is_authenticated
@@ -101,7 +70,7 @@ def user_is_authenticated(user):
 
 def user_is_active(user):
     if user and hasattr(user, 'is_active'):
-        if callable(user.is_active):
+        if isinstance(user.is_active, collections.Callable):
             is_active = user.is_active()
         else:
             is_active = user.is_active
@@ -129,3 +98,20 @@ def first(func, items):
     for item in items:
         if func(item):
             return item
+
+
+def parse_qs(value):
+    """Like urlparse.parse_qs but transform list values to single items"""
+    return drop_lists(battery_parse_qs(value))
+
+
+def drop_lists(value):
+    out = {}
+    for key, val in value.items():
+        val = val[0]
+        if isinstance(key, six.binary_type):
+            key = six.text_type(key, 'utf-8')
+        if isinstance(val, six.binary_type):
+            val = six.text_type(val, 'utf-8')
+        out[key] = val
+    return out
